package ControlledMixingUnit "Package containing example model exploring the use of a non-linear feedforward controller."
  model ControlledMixingUnitExample "Simple example of a mixing unit where a (discretized) nonlinear inverse plant model is used as feedforward controller"
    extends Modelica.Icons.Example;
    import SI = Modelica.Units.SI;
    parameter SI.Frequency freq = 1 / 300 "Critical frequency of filter";
    parameter Real c0(unit = "mol/l") = 0.848 "Nominal concentration";
    parameter SI.Temperature T0 = 308.5 "Nominal temperature";
    parameter Real a1_inv = 0.2674;
    parameter Real a21_inv = 1.815;
    parameter Real a22_inv = 0.4682;
    parameter Real b_inv = 1.5476;
    parameter Real k0_inv = 1.05e14;
    parameter Real eps = 34.2894;
    parameter Real x10 = 0.42;
    parameter Real x10_inv = 0.6;
    parameter Real x20 = 0.01;
    parameter Real u0 = -0.0224;
    final parameter Real c_start(unit = "mol/l") = c0 * (1 - x10);
    final parameter Real c_inv_start(unit = "mol/l") = c0 * (1 - x10_inv);
    final parameter SI.Temperature T_start = T0 * (1 + x20);
    final parameter Real c_high_start(unit = "mol/l") = c0 * (1 - 0.72);
    final parameter Real T_c_start = T0 * (1 + u0);
    parameter Real pro = 1 "Deviations of plant to inverse plant parameters";
    final parameter Real a1 = a1_inv * pro;
    final parameter Real a21 = a21_inv * pro;
    final parameter Real a22 = a22_inv * pro;
    final parameter Real b = b_inv * pro;
    final parameter Real k0 = k0_inv * pro;
    Components.MixingUnit invMixingUnit(c0 = c0, T0 = T0, a1 = a1_inv, a21 = a21_inv, a22 = a22_inv, b = b_inv, k0 = k0_inv, eps = eps, c(start = c_start, fixed = true), T(start = T_start, fixed = true, stateSelect = StateSelect.always), T_c(start = T_c_start)) annotation(Placement(transformation(extent = {{-12, 14}, {-32, 34}})));
    Modelica.Blocks.Math.Add add annotation(Placement(transformation(extent = {{40, -18}, {56, -2}})));
    Modelica.Blocks.Math.InverseBlockConstraints inverseBlockConstraints annotation(Placement(transformation(extent = {{-54, 8}, {-2, 40}})));
    Components.MixingUnit mixingUnit(c(start = c_start, fixed = true), T(start = T_start, fixed = true), c0 = c0, T0 = T0, a1 = a1, a21 = a21, a22 = a22, b = b, k0 = k0, eps = eps) annotation(Placement(transformation(extent = {{88, -20}, {108, 0}})));
    Modelica.Blocks.Math.Feedback feedback annotation(Placement(transformation(extent = {{-24, -20}, {-4, 0}})));
    Modelica.Blocks.Math.Gain gain(k = 20) annotation(Placement(transformation(extent = {{4, -20}, {24, 0}})));
    Components.CriticalDamping filter(n = 3, f = freq, x(start = {0.49, 0.49, 0.49}, fixed = {true, false, false})) annotation(Placement(transformation(extent = {{-86, 14}, {-66, 34}})));
    Modelica.Clocked.RealSignals.Sampler.Hold hold1(y_start = 0) annotation(Placement(transformation(extent = {{66, -16}, {78, -4}})));
    Modelica.Clocked.RealSignals.Sampler.Sample sample1 annotation(Placement(transformation(extent = {{80, -40}, {68, -28}})));
    Modelica.Clocked.ClockSignals.Clocks.PeriodicRealClock periodicClock1(useSolver = true, period = 1, solverMethod = "ExplicitEuler") annotation(Placement(transformation(extent = {{-134, -26}, {-122, -14}})));
    Modelica.Blocks.Sources.Step step(height = c_high_start - c_start, offset = c_start) annotation(Placement(transformation(extent = {{-136, 14}, {-116, 34}})));
    Modelica.Clocked.RealSignals.Sampler.SampleClocked sample2 annotation(Placement(transformation(extent = {{-108, 18}, {-96, 30}})));
  equation
    connect(feedback.y, gain.u) annotation(Line(points = {{-5, -10}, {-5, -10}, {2, -10}}, color = {0, 0, 127}));
    connect(gain.y, add.u2) annotation(Line(points = {{25, -10}, {32, -10}, {32, -14.8}, {38.4, -14.8}}, color = {0, 0, 127}));
    connect(inverseBlockConstraints.y2, invMixingUnit.T_c) annotation(Line(points = {{-5.9, 24}, {-10, 24}}, color = {0, 0, 127}));
    connect(invMixingUnit.c, inverseBlockConstraints.u2) annotation(Line(points = {{-34, 30}, {-38, 30}, {-38, 24}, {-48.8, 24}}, color = {0, 0, 127}));
    connect(invMixingUnit.T, feedback.u1) annotation(Line(points = {{-34, 18}, {-46, 18}, {-46, -10}, {-22, -10}}, color = {0, 0, 127}));
    connect(filter.y, inverseBlockConstraints.u1) annotation(Line(points = {{-65, 24}, {-56.6, 24}}, color = {0, 0, 127}));
    connect(hold1.y, mixingUnit.T_c) annotation(Line(points = {{78.6, -10}, {86, -10}}, color = {0, 0, 127}));
    connect(add.y, hold1.u) annotation(Line(points = {{56.8, -10}, {64.8, -10}}, color = {0, 0, 127}));
    connect(sample1.u, mixingUnit.T) annotation(Line(points = {{81.2, -34}, {116, -34}, {116, -16}, {110, -16}}, color = {0, 0, 127}));
    connect(sample1.y, feedback.u2) annotation(Line(points = {{67.4, -34}, {-14, -34}, {-14, -18}}, color = {0, 0, 127}));
    connect(inverseBlockConstraints.y1, add.u1) annotation(Line(points = {{-0.7, 24}, {30, 24}, {30, -5.2}, {38.4, -5.2}}, color = {0, 0, 127}));
    connect(sample2.u, step.y) annotation(Line(points = {{-109.2, 24}, {-115, 24}}, color = {0, 0, 127}));
    connect(filter.u, sample2.y) annotation(Line(points = {{-88, 24}, {-95.4, 24}}, color = {0, 0, 127}));
    connect(periodicClock1.y, sample2.clock) annotation(Line(points = {{-121.4, -20}, {-102, -20}, {-102, 16.8}}, color = {175, 175, 175}, pattern = LinePattern.Dot, thickness = 0.5));
    annotation(__Wolfram(PlotSet(plots = {Plot(name = "mixingUnit.c", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = mixingUnit.c), Curve(x = time, y = filter.y)}), SubPlot(curves = {Curve(x = time, y = mixingUnit.T_c), Curve(x = time, y = invMixingUnit.T_c), Curve(x = time, y = mixingUnit.T)})})})), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-140, -100}, {120, 100}}), graphics = {Rectangle(extent = {{-90, 44}, {60, -44}}, lineColor = {255, 0, 0}), Text(extent = {{12, 42}, {58, 34}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, textString = "controller")}), experiment(StopTime = 300), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}})), Documentation(info = "<html>


    <p>
A substance A is flowing continuously into a mixing reactor. Due to a catalyst, the substance reacts and splits into several base substances that are continuously removed. The reaction generates energy and therefore the reactor is cooled with a cooling medium. The cooling temperature T_c(t) in [K] is the primary actuation signal. Substance A is described by its concentration c(t) in [mol/l] and its temperature T(t) in [K]. The concentration c(t) is the signal to be primarily controlled and the temperature T(t) is the signal that is measured. These equations are collected together in input/output block Utilities.ComponentsMixingUnit.MixingUnit.
</p>
    <h4>Design of Controller</h4>
    <p>
The controller for the mixing unit is shown in the diagram layer of block at hand, as well as in the following figure:
</p><p>
<img src=\"modelica://ControlledMixingUnit/DocumentationFiles/Diagram.png\" alt=\"Diagram\"/>
</p><p>
It consists of the filter discussed above. The input to the filter is the reference concentration which is filtered by the low pass filter. The output of the filter is used as input to the concentration c in the inverse plant model. This model computes the desired cooling temperature T_c (which is used as desired cooling temperature at the output of the controller) and the desired temperature T (which is used as desired value for the feedback controller). This part of the control system is the feed-forward part that computes the desired actuator signal. As feedback controller a simple P-Controller with one gain is used.
</p><a id=\"headerTag_SimulationResults\">&nbsp;</a>
    <h4>Simulation Results</h4>
    <p>
The overall system presents a good control behavior, as the concentration of the reactant in the mixing unit follows the desired signal.
</p><p>
<img src=\"modelica://ControlledMixingUnit/DocumentationFiles/SimulationResults02.png\" alt=\"SimulationResults02\"/>
</p>

<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://ControlledMixingUnit.ControlledMixingUnitExample/ControlledMixingUnit.nb\">ControlledMixingUnit.nb</a>.</p>
</html>"));
  end ControlledMixingUnitExample;

  package Components "Package containing the components of the example model"
    extends Modelica.Icons.Package;

    model MixingUnit "Mixing unit demo from Foellinger, Nichtlineare Regelungen II, p. 280"
      import SI = Modelica.Units.SI;
      Modelica.Blocks.Interfaces.RealInput T_c(unit = "K") "Cooling temperature" annotation(Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      Modelica.Blocks.Interfaces.RealOutput c(unit = "mol/l") "Concentration" annotation(Placement(transformation(extent = {{100, 40}, {140, 80}})));
      Modelica.Blocks.Interfaces.RealOutput T(unit = "K") "Temperature in mixing unit" annotation(Placement(transformation(extent = {{100, -80}, {140, -40}})));
      parameter Real c0(unit = "mol/l") = 0.848 "Nominal concentration";
      parameter SI.Temperature T0 = 308.5 "Nominal temperature";
      parameter Real a1 = 0.2674;
      parameter Real a21 = 1.815;
      parameter Real a22 = 0.4682;
      parameter Real b = 1.5476;
      parameter Real k0 = 1.05e14;
      parameter Real eps = 34.2894;
      Real gamma "Reaction speed";
    protected
      parameter SI.Time tau0 = 60;
      parameter Real wk0 = k0 / c0;
      parameter Real weps = eps * T0;
      parameter Real wa11 = a1 / tau0;
      parameter Real wa12 = c0 / tau0;
      parameter Real wa13 = c0 * a1 / tau0;
      parameter Real wa21 = a21 / tau0;
      parameter Real wa22 = a22 * T0 / tau0;
      parameter Real wa23 = T0 * (a21 - b) / tau0;
      parameter Real wb = b / tau0;
    equation
      gamma = c * wk0 * exp(-weps / T);
      der(c) = -wa11 * c - wa12 * gamma + wa13;
      der(T) = -wa21 * T + wa22 * gamma + wa23 + wb * T_c;
      annotation(Documentation(info = "<html>

    <p>
This is a simple non-linear plant model of a mixing unit based on the work of  Foellinger, Nichtlineare Regelungen II, p. 280
</p></html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}})), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{-100, -100}, {100, 100}}), graphics = {Rectangle(extent = {{-100, 40}, {100, -100}}, lineColor = {255, 255, 255}, fillColor = {0, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{-100, 100}, {-100, -100}, {100, -100}, {100, 100}}, color = {0, 0, 0}), Text(extent = {{-144, 148}, {148, 100}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, textString = "%name"), Text(extent = {{118, -86}, {160, -124}}, textString = "T"), Text(extent = {{116, 114}, {158, 76}}, textString = "c"), Text(extent = {{-194, 72}, {-108, 34}}, textString = "T_c"), Line(points = {{0, -50}, {0, -100}}), Ellipse(extent = {{-42, -38}, {0, -66}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Ellipse(extent = {{0, -38}, {42, -66}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Line(points = {{0, 80}, {0, 16}}), Line(points = {{20, 80}, {20, 16}}), Line(points = {{-86, -72}, {-86, -114}}), Line(points = {{-66, -72}, {-66, -114}})}));
    end MixingUnit;

    block CriticalDamping "Output the input signal filtered with an n-th order filter with critical damping"
      import Modelica.Blocks.Types.Init;
      extends Modelica.Blocks.Interfaces.SISO;
      parameter Integer n = 2 "Order of filter";
      parameter Modelica.Units.SI.Frequency f(start = 1) "Cut-off frequency";
      parameter Boolean normalized = true "= true, if amplitude at f_cut is 3 dB, otherwise unmodified filter";
      /*
  parameter Real x_start[n]=zeros(n) "Initial or guess values of states"
    annotation (Dialog(group="Initialization"));
*/
      output Real x[n](start = zeros(n)) "Filter states";
    protected
      parameter Real alpha = if normalized then sqrt(2 ^ (1 / n) - 1) else 1.0 "Frequency correction factor for normalized filter";
      parameter Real w = 2 * Modelica.Constants.pi * f / alpha;
    equation
      der(x[1]) = (u - x[1]) * w;
      for i in 2:n loop
        der(x[i]) = (x[i - 1] - x[i]) * w;
      end for;
      y = x[n];
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{-80.6897, 77.6256}, {-80.6897, -90.3744}}, color = {192, 192, 192}), Polygon(points = {{-79.7044, 90.6305}, {-87.7044, 68.6305}, {-71.7044, 68.6305}, {-79.7044, 88.6305}, {-79.7044, 90.6305}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Line(points = {{-90, -80}, {82, -80}}, color = {192, 192, 192}), Polygon(points = {{90, -80}, {68, -72}, {68, -88}, {90, -80}}, lineColor = {192, 192, 192}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid), Text(extent = {{0, 0}, {60, -60}}, lineColor = {192, 192, 192}, textString = "PTn"), Line(points = {{-80.7599, -80.5082}, {-70.7599, -74.5082}, {-56, -60}, {-48, -42}, {-42, -18}, {-36, 4}, {-26, 20}, {-10.7599, 34.9018}, {-0.759907, 38.8218}, {9.24009, 41.6818}, {19.2401, 43.7818}, {29.2401, 45.3118}, {39.2401, 46.4318}, {49.2401, 47.2518}, {59.2401, 47.8518}, {69.2401, 48.2918}, {79.2401, 48.6118}}, color = {0, 0, 127}), Text(extent = {{-70, 94}, {26, 48}}, lineColor = {192, 192, 192}, textString = "%n"), Text(extent = {{8, -106}, {8, -146}}, textString = "f=%f")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Line(points = {{40, 0}, {-40, 0}}), Text(extent = {{-55, 55}, {55, 5}}, textString = "1"), Rectangle(extent = {{-60, 60}, {60, -60}}, lineColor = {0, 0, 255}), Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 255}), Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 255}), Text(extent = {{-54, -6}, {44, -56}}, textString = "(s/w + 1)"), Text(extent = {{38, -10}, {58, -30}}, textString = "n")}), Documentation(info = "<html>

    <p>
This block defines the transfer function between the input u and the output y as an n-th order filter with critical damping characteristics and cut-off frequency f.
</p></html>"));
    end CriticalDamping;
    annotation(Documentation);
  end Components;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0")), __Wolfram(openClass = ControlledMixingUnit.ControlledMixingUnitModel), Documentation, Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end ControlledMixingUnit;
